@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1999 - 2001	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1999 - 2004	David Albert Bagley, bagleyd@tux.org
 #
 #  Abacus demo and neat pointers from
 #  Copyright (c) 1991 - 1998	Luis Fernandes, elf@ee.ryerson.ca
@@ -28,6 +28,9 @@
 
 /* Methods demo file for Abacus */
 
+#if WINVER
+#include "AbacusP.h"
+#else
 #include <stdlib.h>
 #include <stdio.h>
 #ifdef VMS
@@ -54,41 +57,47 @@ static void DestroyAbacusDemo(Widget old);
 static Boolean SetValuesAbacusDemo(Widget current, Widget request, Widget renew);
 
 static void QuitAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs);
+static void HideAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs);
 static void SelectAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs);
+static void ClearAbacusDemo(AbacusWidget w);
+static void DemoAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs);
 static void NextAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs);
 static void RepeatAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs);
 static void MoreAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs);
 
-static void SetAllColors(AbacusWidget w, Boolean init);
-static void ClearDemo(AbacusWidget w);
-static void QueryDemo(AbacusWidget w, Boolean advance);
-static void IncDemo(AbacusWidget w);
-
 static char defaultTranslationsAbacusDemo[] =
 "<KeyPress>q: quit()\n\
-   Ctrl<KeyPress>C: quit()\n\
-   <Btn1Down>: select()\n\
-   <KeyPress>c: select()\n\
-   <KeyPress>n: next()\n\
-   <KeyPress>r: repeat()\n\
-   <KeyPress>0x20: more()\n\
-   <KeyPress>0x40: more()\n\
-   <KeyPress>KP_Space: more()\n\
-   <KeyPress>Return: more()";
+ Ctrl<KeyPress>C: quit()\n\
+ <KeyPress>osfCancel: hide()\n\
+ <KeyPress>Escape: hide()\n\
+ <KeyPress>osfEscape: hide()\n\
+ Ctrl<KeyPress>[: hide()\n\
+ <KeyPress>0x1B: hide()\n\
+ <Btn1Down>: select()\n\
+ <KeyPress>c: select()\n\
+ <KeyPress>o: demo()\n\
+ <KeyPress>n: next()\n\
+ <KeyPress>r: repeat()\n\
+ <KeyPress>0x20: more()\n\
+ <KeyPress>KP_Space: more()\n\
+ <KeyPress>Return: more()";
 
 /* KP_Space does not work here
    0x20 is SP (' ') in ASCII  (DP in EBCDIC)
-   0x40 is SP (' ') in EBCDIC ('@' in ASCII) 
+   0x40 is SP (' ') in EBCDIC ('@' in ASCII)
+ <KeyPress>0x27: hide()\n\
+ <KeyPress>0x40: more()\n\
  */
 
-
 static XtActionsRec actionsListAbacusDemo[] =
 {
-	{"quit", (XtActionProc) QuitAbacusDemo},
-	{"select", (XtActionProc) SelectAbacusDemo},
-	{"next", (XtActionProc) NextAbacusDemo},
-	{"repeat", (XtActionProc) RepeatAbacusDemo},
-	{"more", (XtActionProc) MoreAbacusDemo}
+	{(char *) "quit", (XtActionProc) QuitAbacusDemo},
+	{(char *) "hide", (XtActionProc) HideAbacusDemo},
+	{(char *) "select", (XtActionProc) SelectAbacusDemo},
+	{(char *) "demo", (XtActionProc) DemoAbacusDemo},
+	{(char *) "next", (XtActionProc) NextAbacusDemo},
+	{(char *) "repeat", (XtActionProc) RepeatAbacusDemo},
+	{(char *) "more", (XtActionProc) MoreAbacusDemo}
 };
 
 static XtResource resourcesAbacusDemo[] =
@@ -129,6 +138,9 @@ static XtResource resourcesAbacusDemo[] =
 	{XtNframed, XtCFramed, XtRBoolean, sizeof (Boolean),
 	 XtOffset(AbacusWidget, abacusDemo.framed),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNmode, XtCMode, XtRInt, sizeof (int),
+	 XtOffset(AbacusWidget, abacus.mode),
+	 XtRString, (caddr_t) "5"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(AbacusWidget, abacus.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -138,7 +150,7 @@ AbacusClassRec abacusDemoClassRec =
 {
 	{
 		(WidgetClass) & widgetClassRec,		/* superclass */
-		"AbacusDemo",	/* class name */
+		(char *) "AbacusDemo",	/* class name */
 		sizeof (AbacusRec),	/* widget size */
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
@@ -181,14 +193,91 @@ AbacusClassRec abacusDemoClassRec =
 
 WidgetClass abacusDemoWidgetClass = (WidgetClass) & abacusDemoClassRec;
 
-#define LINES 4
-#define CHARS 64
+static void
+SetAllColors(AbacusWidget w, Boolean init)
+{
+	XGCValues   values;
+	XtGCMask    valueMask;
+
+	valueMask = GCForeground | GCBackground;
+	if (w->abacus.mono) {
+		if (w->abacus.reverse) {
+			values.background = w->abacus.foreground;
+			values.foreground = w->core.background_pixel;
+		} else {
+			values.foreground = w->abacus.foreground;
+			values.background = w->core.background_pixel;
+		}
+	} else {
+		if (w->abacus.reverse) {
+			values.foreground = w->abacusDemo.background;
+			values.background = w->abacusDemo.foreground;
+		} else {
+			values.background = w->abacusDemo.background;
+			values.foreground = w->abacusDemo.foreground;
+		}
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->abacusDemo.foregroundGC);
+	w->abacusDemo.foregroundGC = XtGetGC((Widget) w, valueMask, &values);
+
+	if (w->abacus.mono) {
+		if (w->abacus.reverse) {
+			values.foreground = w->abacus.foreground;
+			values.background = w->core.background_pixel;
+		} else {
+			values.background = w->abacus.foreground;
+			values.foreground = w->core.background_pixel;
+		}
+	} else {
+		if (w->abacus.reverse) {
+			values.background = w->abacusDemo.background;
+			values.foreground = w->abacusDemo.foreground;
+		} else {
+			values.foreground = w->abacusDemo.background;
+			values.background = w->abacusDemo.foreground;
+		}
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->abacusDemo.inverseGC);
+	w->abacusDemo.inverseGC = XtGetGC((Widget) w, valueMask, &values);
+}
+
+static void
+loadFont(AbacusWidget w)
+{
+	/*const char *fontname = "-*-times-*-r-*-*-*-180-*"; */
+	const char       *altfontname = "8x13";
+	char buf[512];
+
+	/* Access font */
+	if ((w->abacusDemo.fontInfo = XLoadQueryFont(XtDisplay(w),
+					      w->abacusDemo.font)) == NULL) {
+		(void) sprintf(buf,
+			"Cannot open %s font.\nAttempting %s font as alternate\n",
+			w->abacusDemo.font, altfontname);
+		DISPLAY_WARNING(buf);
+		if ((w->abacusDemo.fontInfo = XLoadQueryFont(XtDisplay(w),
+						     altfontname)) == NULL) {
+			(void) sprintf(buf,
+				"Cannot open %s alternate font.\nUse the -demofont option to specify a font to use.\n",
+				altfontname);
+			DISPLAY_ERROR(buf);
+		}
+	}
+	w->abacusDemo.fontHeight = w->abacusDemo.fontInfo->max_bounds.ascent +
+		w->abacusDemo.fontInfo->max_bounds.descent;
+}
+
+#endif
+
+#define SetAbacusDemo(w,r) SetAbacus(w,r)
 
 typedef enum {
 	INTRO, QUERY, DISPLAY, CONCL
 } texttypes;
 
-static char *introframelesstext[LINES] =
+static const char *introframelesstext[LINES] =
 {
 	"Place this window below the abacus, then click & leave",
 	"the pointer in the abacus window to begin the demo.",
@@ -196,7 +285,7 @@ static char *introframelesstext[LINES] =
 	"Type `q' to quit the demo.",
 };
 
-static char *introframedtext[LINES] =
+static const char *introframedtext[LINES] =
 {
 	"Click & leave the pointer in the abacus window",
 	"to begin the demo.",
@@ -204,7 +293,7 @@ static char *introframedtext[LINES] =
 	"Type `q' to quit the demo.",
 };
 
-static char *querytext[LINES] =
+static const char *querytext[LINES] =
 {
 	"Type:",
 	"  `n' for the Next lesson",
@@ -212,7 +301,7 @@ static char *querytext[LINES] =
 	"  `q' to Quit the demo",
 };
 
-static char *concltext[LINES] =
+static const char *concltext[LINES] =
 {
 	"Here Endth The Lesson",
 	"",
@@ -226,48 +315,25 @@ static char *concltext[LINES] =
 static char displaytext[LINES][CHARS];
 
 static void
-loadFont(AbacusWidget w)
-{
-	/*char *fontname = "-*-times-*-r-*-*-*-180-*"; */
-	char       *altfontname = "8x13";
-
-	/* Access font */
-	if ((w->abacusDemo.fontInfo = XLoadQueryFont(XtDisplay(w),
-					      w->abacusDemo.font)) == NULL) {
-		(void) fprintf(stderr, "Cannot open %s font\n",
-			       w->abacusDemo.font);
-		(void) fprintf(stderr, "Attempting %s font as alternate\n",
-			       altfontname);
-
-		if ((w->abacusDemo.fontInfo = XLoadQueryFont(XtDisplay(w),
-						     altfontname)) == NULL) {
-			(void) fprintf(stderr,
-				       "Cannot open %s alternate font\n",
-				       altfontname);
-			(void) fprintf(stderr,
-				       "use the -demofont option to specify a font to use\n");
-			exit(-1);
-		}
-	}
-	w->abacusDemo.fontHeight = w->abacusDemo.fontInfo->max_bounds.ascent +
-		w->abacusDemo.fontInfo->max_bounds.descent;
-}
-
-static void
-drawText(AbacusWidget w, GC gc, texttypes textt)
+drawText(AbacusWidget w, texttypes textt)
 {
 	int         line;
-	static Bool firstTime = True;
-
+	static Boolean firstTime = True;
 	if (!firstTime) {
-		XFillRectangle(XtDisplay(w), XtWindow(w), w->abacusDemo.inverseGC,
-			       0, 0, w->core.width, w->core.height);
+#ifdef WINVER
+		FILLRECTANGLE(w, w->abacus.inverseGC,
+			0, w->core.height, w->core.width,
+			w->core.height + LINES * w->abacusDemo.fontHeight - 1);
+#else
+		FILLRECTANGLE(w, w->abacusDemo.inverseGC,
+			0, 0, w->core.width, w->core.height);
+#endif
 	} else {
 		firstTime = False;
 	}
 
 	for (line = 0; line < w->abacusDemo.lines; line++) {
-		const char  *temp;
+		const char * temp;
 
 		switch (textt) {
 			case INTRO:
@@ -303,10 +369,32 @@ drawText(AbacusWidget w, GC gc, texttypes textt)
 #ifdef DEBUG
 		(void) printf("%d, %d %s\n", w->abacusDemo.fontHeight, line, temp);
 #endif
-		XDrawString(XtDisplay(w), XtWindow(w), gc,
-			    1, w->abacusDemo.fontHeight + (w->abacusDemo.fontHeight * line),
-			    temp, strlen(temp));
+#ifdef WINVER
+		(void) SetTextColor(w->core.hDC, w->abacusDemo.foregroundGC);
+		(void) SetBkMode(w->core.hDC, TRANSPARENT);
+		(void) TextOut(w->core.hDC,
+			1, w->core.height + w->abacusDemo.fontHeight * line - 1,
+			temp, (signed) strlen(temp));
+#else
+		(void) XDrawString(XtDisplay(w), XtWindow(w),
+			w->abacusDemo.foregroundGC,
+			1, w->abacusDemo.fontHeight + (w->abacusDemo.fontHeight * line),
+			temp, strlen(temp));
 		XFlush(XtDisplay(w));
+#endif
+	}
+}
+
+char *
+postfixType(int mode)
+{
+	switch (mode) {
+	case CHINESE: return "";
+	case JAPANESE: return "jp";
+	case KOREAN: return "ko";
+	case ROMAN: return "jp"; /* ro */
+	case RUSSIAN: return "ru";
+	default: return "err";
 	}
 }
 
@@ -314,20 +402,27 @@ drawText(AbacusWidget w, GC gc, texttypes textt)
 static void
 setLessons(AbacusWidget w)
 {
-	static char fname[512];
+	char fname[512], lname[512];
 
 	w->abacusDemo.lessons = 0;
 	for (;;) {
 		w->abacusDemo.lessons++;
-		(void) sprintf(fname, "%s/Lesson%d.cmd",
-			  (w->abacusDemo.path[0]) ? w->abacusDemo.path : ".",
-			       w->abacusDemo.lessons);
+		(void) sprintf(fname, "%s%sAbacus%s%d.les", w->abacusDemo.path,
+			FINALDELIM, postfixType(w->abacus.mode),
+			w->abacusDemo.lessons);
 		if ((w->abacusDemo.fp = fopen(fname, "r")) == NULL) {
-			w->abacusDemo.lessons--;
+			/* Try current directory in case its not installed yet. */
+			(void) sprintf(lname, "%sAbacus%s%d.les", CURRENTDELIM,
+				postfixType(w->abacus.mode),
+				w->abacusDemo.lessons);
+			if ((w->abacusDemo.fp = fopen(lname, "r")) == NULL) {
+				w->abacusDemo.lessons--;
 #ifdef DEBUG
-			(void) printf("lessons = %d\n", w->abacusDemo.lessons);
+				(void) printf("lessons = %d\n",
+					w->abacusDemo.lessons);
 #endif
-			return;
+				return;
+			}
 		}
 		(void) fclose(w->abacusDemo.fp);
 	}
@@ -336,22 +431,31 @@ setLessons(AbacusWidget w)
 static void
 startLesson(AbacusWidget w)
 {
-	static char fname[512];
+	char fname[512], lname[512], buf[512];
 
-	(void) sprintf(fname, "%s/Lesson%d.cmd",
-		       (w->abacusDemo.path[0]) ? w->abacusDemo.path : ".",
-		       w->abacusDemo.lessonCount + 1);
+	(void) sprintf(fname, "%s%sAbacus%s%d.les", w->abacusDemo.path,
+		FINALDELIM, postfixType(w->abacus.mode),
+		w->abacusDemo.lessonCount + 1);
 
 #ifdef DEBUG
-	printf("Lesson name = %s\n", fname);
+	(void) printf("Lesson name = %s\n", fname);
 #endif
 	if ((w->abacusDemo.fp = fopen(fname, "r")) == NULL) {
-		(void) fprintf(stderr,
-		    "Error, could not open lesson script-file: %s\n", fname);
-		exit(1);
+
+		/* Try current directory in case its not installed yet. */
+		(void) sprintf(lname, "%sAbacus%s%d.les", CURRENTDELIM,
+		       postfixType(w->abacus.mode),
+		       w->abacusDemo.lessonCount + 1);
+		if ((w->abacusDemo.fp = fopen(lname, "r")) == NULL) {
+			(void) sprintf(buf,
+				"Could not open lesson script-file: %s\n",
+				fname);
+			DISPLAY_ERROR(buf);
+		}
 	}
 	w->abacusDemo.lessonLength = 0;
-	while (fscanf(w->abacusDemo.fp, "%d %d %d %d\n",
+	while (fscanf(w->abacusDemo.fp, "%d %d %d %d %d\n",
+		  &(w->abacusDemo.aux),
 		  &(w->abacusDemo.deck), &(w->abacusDemo.rail),
 		  &(w->abacusDemo.number), &(w->abacusDemo.lines)) > 0) {
 		int         line;
@@ -365,10 +469,18 @@ startLesson(AbacusWidget w)
 		++w->abacusDemo.lessonLength;
 	}
 	(void) fclose(w->abacusDemo.fp);
+	/* rewind */
 	if ((w->abacusDemo.fp = fopen(fname, "r")) == NULL) {
-		(void) fprintf(stderr,
-		    "Error, could not open lesson script-file: %s\n", fname);
-		exit(1);
+		/* Try current directory in case its not installed yet. */
+		(void) sprintf(lname, "%sAbacus%s%d.les", CURRENTDELIM,
+			postfixType(w->abacus.mode),
+			w->abacusDemo.lessonCount + 1);
+		if ((w->abacusDemo.fp = fopen(lname, "r")) == NULL) {
+			(void) sprintf(buf,
+				"Could not open lesson script-file: %s\n",
+				fname);
+			DISPLAY_ERROR(buf);
+		}
 	}
 #ifdef DEBUG
 	(void) printf("lessonLength = %d\n", w->abacusDemo.lessonLength);
@@ -376,18 +488,20 @@ startLesson(AbacusWidget w)
 }
 
 static void
-doLesson(AbacusWidget w, GC gc)
+doLesson(AbacusWidget w)
 {
 	int         line;
 
 	/* load first instance */
-	(void) fscanf(w->abacusDemo.fp, "%d %d %d %d\n",
-		      &(w->abacusDemo.deck), &(w->abacusDemo.rail),
-		      &(w->abacusDemo.number), &(w->abacusDemo.lines));
+	(void) fscanf(w->abacusDemo.fp, "%d %d %d %d %d\n",
+		&(w->abacusDemo.aux),
+		&(w->abacusDemo.deck), &(w->abacusDemo.rail),
+		&(w->abacusDemo.number), &(w->abacusDemo.lines));
 #ifdef DEBUG
-	(void) printf("deck = %d, rail = %d, number = %d, lines = %d\n",
-		      w->abacusDemo.deck, w->abacusDemo.rail,
-		      w->abacusDemo.number, w->abacusDemo.lines);
+	(void) printf("aux %d, deck = %d, rail = %d, number = %d, lines = %d\n",
+		w->abacusDemo.aux,
+		w->abacusDemo.deck, w->abacusDemo.rail,
+		w->abacusDemo.number, w->abacusDemo.lines);
 #endif
 
 	if  (w->abacusDemo.lines > LINES)
@@ -402,81 +516,90 @@ doLesson(AbacusWidget w, GC gc)
 	 */
 
 	if (w->abacusDemo.lessonLength) {
-		abacusCallbackStruct cb;
-
-		cb.reason = ABACUS_MOVE;
-		cb.deck = w->abacusDemo.deck;
-		cb.rail = w->abacusDemo.rail;
-		cb.number = w->abacusDemo.number;
-		XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
+		SetAbacusMove(w, ABACUS_MOVE, w->abacusDemo.aux,
+			w->abacusDemo.deck, w->abacusDemo.rail,
+			w->abacusDemo.number);
 	}
-	drawText(w, gc, DISPLAY);
+	drawText(w, DISPLAY);
 
 	if (--w->abacusDemo.lessonLength < 0) {
 		(void) fclose(w->abacusDemo.fp);
 		w->abacusDemo.lines = LINES;
-		drawText(w, gc, CONCL);
-		(void) sleep(1);
-		drawText(w, gc, QUERY);
+		drawText(w, CONCL);
+		Sleep(1000);
+		drawText(w, QUERY);
 	}
 }
 
 static void
-InitializeAbacusDemo(Widget request, Widget renew)
+ClearDemo(AbacusWidget w)
 {
+        SetAbacusDemo(w, ABACUS_CLEAR);
+	ClearAbacusDemo(w);
+}
+
+static void
+QueryDemo(AbacusWidget w, Boolean advance)
+{
+	if (w->abacusDemo.query) {
+		w->abacusDemo.started = True;
+		w->abacusDemo.query = False;
+		if (w->abacusDemo.lessons && advance)
+			w->abacusDemo.lessonCount =
+				(w->abacusDemo.lessonCount + 1) %
+				w->abacusDemo.lessons;
+		SetAbacusDemo(w, ABACUS_CLEAR);
+#ifdef WINVER
+		ClearAbacus(w);
+#endif
+		startLesson(w);
+		doLesson(w);
+	}
+}
+
+static void
+IncDemo(AbacusWidget w)
+{
+	if (w->abacusDemo.started && !w->abacusDemo.query) {
+		w->abacusDemo.query = False;
+		doLesson(w);
+	}
+}
+
+#ifndef WINVER
+static
+#endif
+void
+InitializeAbacusDemo(
+#ifdef WINVER
+	AbacusWidget w
+#else
+	Widget request, Widget renew
+#endif
+	)
+{
+#ifndef WINVER
 	AbacusWidget w = (AbacusWidget) renew;
 
-	w->abacus.depth = DefaultDepthOfScreen(XtScreen(w));
+	w->abacus.mono = (DefaultDepthOfScreen(XtScreen(w)) < 2 ||
+		w->abacus.mono);
 	SetAllColors(w, True);
-	w->abacusDemo.started = False;
-	w->abacusDemo.query = False;
-	w->abacusDemo.lessonCount = 0;
-
+	loadFont(w);
 #if 0
 	ResizeAbacusDemo(w);
 #endif
-	setLessons(w);
-	loadFont(w);
-}
-
-static void
-DestroyAbacusDemo(Widget old)
-{
-	AbacusWidget w = (AbacusWidget) old;
-
-	XtReleaseGC(old, w->abacusDemo.foregroundGC);
-	XtReleaseGC(old, w->abacusDemo.inverseGC);
-	XtRemoveCallbacks(old, XtNselectCallback, w->abacus.select);
-}
-
+#else
 #if 0
-static void
-ResizeAbacusDemo(AbacusWidget w)
-{
-}
-
+	hBrush = CreateSolidBrush(w->abacusDemo.background);
+	SETBACK(w->core.hWnd, hBrush);
 #endif
-
-static void
-ExposeAbacusDemo(Widget renew, XEvent * event, Region region)
-{
-	AbacusWidget w = (AbacusWidget) renew;
-
-	if (w->core.visible) {
-		XFillRectangle(XtDisplay(w), XtWindow(w),
-			       w->abacusDemo.inverseGC, 0, 0, w->core.width, w->core.height);
-		if (w->abacusDemo.started) {
-			drawText(w, w->abacusDemo.foregroundGC, DISPLAY);
-		} else if (w->abacusDemo.query) {
-			w->abacusDemo.lines = LINES;
-			drawText(w, w->abacusDemo.foregroundGC, QUERY);
-		} else {
-			w->abacusDemo.lines = LINES;
-			drawText(w, w->abacusDemo.foregroundGC, INTRO);
-		}
-	}
+#endif
+	w->abacusDemo.started = False;
+	w->abacusDemo.query = False;
+	w->abacusDemo.lessonCount = 0;
 }
 
+#ifndef WINVER
 static      Boolean
 SetValuesAbacusDemo(Widget current, Widget request, Widget renew)
 {
@@ -488,6 +611,10 @@ SetValuesAbacusDemo(Widget current, Widget request, Widget renew)
 
 #endif
 
+	if (w->abacus.mode != c->abacus.mode) {
+		setLessons(w);
+		ClearAbacusDemo(w);
+	}
 	if (w->core.background_pixel != c->core.background_pixel ||
 	    w->abacus.foreground != c->abacus.foreground ||
 	    w->abacusDemo.background != c->abacusDemo.background ||
@@ -532,115 +659,162 @@ QuitAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs)
 }
 
 static void
-SelectAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs)
+DestroyAbacusDemo(Widget old)
+{
+	AbacusWidget w = (AbacusWidget) old;
+
+	XtReleaseGC(old, w->abacusDemo.foregroundGC);
+	XtReleaseGC(old, w->abacusDemo.inverseGC);
+	XtRemoveCallbacks(old, XtNselectCallback, w->abacus.select);
+	XFreeFont(XtDisplay(w), w->abacusDemo.fontInfo);
+}
+
+#else
+#if 0
+void
+DestroyAbacusDemo(HBRUSH hBrush)
+{
+	(void) DeleteObject(hBrush);
+	PostQuitMessage((int) NULL);
+}
+#endif
+#endif
+
+#if 0
+static void
+ResizeAbacusDemo(AbacusWidget w)
+{
+}
+#endif
+
+#ifndef WINVER
+static
+#endif
+void
+ExposeAbacusDemo(
+#ifdef WINVER
+AbacusWidget w
+#else
+Widget renew, XEvent * event, Region region
+#endif
+)
+{
+#ifdef WINVER
+	FILLRECTANGLE(w, w->abacus.inverseGC,
+		0, w->core.height, w->core.width,
+		w->core.height + LINES * w->abacusDemo.fontHeight - 1);
+#else
+	AbacusWidget w = (AbacusWidget) renew;
+
+	if (!w->core.visible)
+		return;
+	FILLRECTANGLE(w, w->abacusDemo.inverseGC,
+		0, 0, w->core.width, w->core.height);
+#endif
+	if (w->abacusDemo.started) {
+		drawText(w, DISPLAY);
+	} else if (w->abacusDemo.query) {
+		w->abacusDemo.lines = LINES;
+		drawText(w, QUERY);
+	} else {
+		w->abacusDemo.lines = LINES;
+		drawText(w, INTRO);
+	}
+}
+
+#ifndef WINVER
+static
+#endif
+void
+HideAbacusDemo(AbacusWidget w
+#ifndef WINVER
+, XEvent * event, char **args, int nArgs
+#endif
+	)
+{
+        SetAbacusDemo(w, ABACUS_HIDE);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SelectAbacusDemo(AbacusWidget w
+#ifndef WINVER
+, XEvent * event, char **args, int nArgs
+#endif
+	)
 {
 	ClearDemo(w);
 }
 
-static void
-NextAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs)
+#ifndef WINVER
+static
+#endif
+void
+ClearAbacusDemo(AbacusWidget w)
+{
+	w->abacusDemo.started = True;
+	w->abacusDemo.query = False;
+	startLesson(w);
+	doLesson(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+DemoAbacusDemo(AbacusWidget w
+#ifndef WINVER
+, XEvent * event, char **args, int nArgs
+#endif
+	)
+{
+#ifdef WINVER
+	w->abacus.demo = !w->abacus.demo;
+	if (w->abacus.demo)
+        	SetAbacusDemo(w, ABACUS_DEMO_DEFAULT);
+			
+#else
+        SetAbacusDemo(w, ABACUS_DEMO);
+#endif
+}
+
+#ifndef WINVER
+static
+#endif
+void
+NextAbacusDemo(AbacusWidget w
+#ifndef WINVER
+, XEvent * event, char **args, int nArgs
+#endif
+	)
 {
 	QueryDemo(w, True);
 }
 
-static void
-RepeatAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs)
+#ifndef WINVER
+static
+#endif
+void
+RepeatAbacusDemo(AbacusWidget w
+#ifndef WINVER
+, XEvent * event, char **args, int nArgs
+#endif
+	)
 {
 	QueryDemo(w, False);
 }
 
-static void
-MoreAbacusDemo(AbacusWidget w, XEvent * event, char **args, int nArgs)
+#ifndef WINVER
+static
+#endif
+void
+MoreAbacusDemo(AbacusWidget w
+#ifndef WINVER
+, XEvent * event, char **args, int nArgs
+#endif
+	)
 {
 	IncDemo(w);
 }
-
-static void
-SetAllColors(AbacusWidget w, Boolean init)
-{
-	XGCValues   values;
-	XtGCMask    valueMask;
-
-	valueMask = GCForeground | GCBackground;
-	if (w->abacus.depth < 2 || w->abacus.mono) {
-		if (w->abacus.reverse) {
-			values.background = w->abacus.foreground;
-			values.foreground = w->core.background_pixel;
-		} else {
-			values.foreground = w->abacus.foreground;
-			values.background = w->core.background_pixel;
-		}
-	} else {
-		if (w->abacus.reverse) {
-			values.foreground = w->abacusDemo.background;
-			values.background = w->abacusDemo.foreground;
-		} else {
-			values.background = w->abacusDemo.background;
-			values.foreground = w->abacusDemo.foreground;
-		}
-	}
-	if (!init)
-		XtReleaseGC((Widget) w, w->abacusDemo.foregroundGC);
-	w->abacusDemo.foregroundGC = XtGetGC((Widget) w, valueMask, &values);
-
-	if (w->abacus.depth < 2 || w->abacus.mono) {
-		if (w->abacus.reverse) {
-			values.foreground = w->abacus.foreground;
-			values.background = w->core.background_pixel;
-		} else {
-			values.background = w->abacus.foreground;
-			values.foreground = w->core.background_pixel;
-		}
-	} else {
-		if (w->abacus.reverse) {
-			values.background = w->abacusDemo.background;
-			values.foreground = w->abacusDemo.foreground;
-		} else {
-			values.foreground = w->abacusDemo.background;
-			values.background = w->abacusDemo.foreground;
-		}
-	}
-	if (!init)
-		XtReleaseGC((Widget) w, w->abacusDemo.inverseGC);
-	w->abacusDemo.inverseGC = XtGetGC((Widget) w, valueMask, &values);
-}
-
-static void
-ClearDemo(AbacusWidget w)
-{
-	abacusCallbackStruct cb;
-
-	w->abacusDemo.started = True;
-	w->abacusDemo.query = False;
-	cb.reason = ABACUS_CLEAR;
-	XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
-	startLesson(w);
-	doLesson(w, w->abacusDemo.foregroundGC);
-}
-
-static void
-QueryDemo(AbacusWidget w, Boolean advance)
-{
-	if (w->abacusDemo.query) {
-		abacusCallbackStruct cb;
-
-		w->abacusDemo.started = True;
-		w->abacusDemo.query = False;
-		if (w->abacusDemo.lessons && advance)
-			w->abacusDemo.lessonCount =
-				(w->abacusDemo.lessonCount + 1) % w->abacusDemo.lessons;
-		cb.reason = ABACUS_CLEAR;
-		XtCallCallbacks((Widget) w, XtNselectCallback, &cb);
-		startLesson(w);
-		doLesson(w, w->abacusDemo.foregroundGC);
-	}
-}
-
-static void
-IncDemo(AbacusWidget w)
-{
-	if (w->abacusDemo.started && !w->abacusDemo.query) {
-		w->abacusDemo.query = False;
-		doLesson(w, w->abacusDemo.foregroundGC);
-	}
-}
